/*
 * Ibis - IRCv3 Library
 * Copyright (C) 2022-2025 Ibis Developers
 *
 * Ibis is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, see <https://www.gnu.org/licenses/>.
 */

#include <stdlib.h>

#include "ibisdcc.h"

#include "ibisstring.h"

/******************************************************************************
 * Public API
 *****************************************************************************/
GInetAddress *
ibis_dcc_host_parse(const char *host) {
	GInetAddress *address = NULL;
	guint32 value = 0;

	g_return_val_if_fail(!ibis_str_is_empty(host), NULL);

	/* Attempt to create the address directly from the string. */
	address = g_inet_address_new_from_string(host);
	if(G_IS_INET_ADDRESS(address)) {
		return address;
	}

	/* That didn't work, so let's see if this is an integer that we can turn
	 * into an IPv4 address.
	 */
	value = g_htonl(atoi(host));
	address = g_inet_address_new_from_bytes((const guint8 *)&value,
	                                        G_SOCKET_FAMILY_IPV4);
	if(G_IS_INET_ADDRESS(address)) {
		return address;
	}

	return NULL;
}

char *
ibis_dcc_host_to_string(GInetAddress *address) {
	GSocketFamily family = G_SOCKET_FAMILY_INVALID;

	g_return_val_if_fail(G_IS_INET_ADDRESS(address), NULL);

	family = g_inet_address_get_family(address);
	switch(family) {
	case G_SOCKET_FAMILY_IPV4: {
		const guint8 *bytes = NULL;

		bytes = g_inet_address_to_bytes(address);
		if(g_inet_address_get_native_size(address) == 4) {
			guint32 *network = (guint32 *)bytes;
			return g_strdup_printf("%u", g_ntohl(*network));
		}

		break;
	}
	case G_SOCKET_FAMILY_IPV6:
		return g_inet_address_to_string(address);
		break;
	case G_SOCKET_FAMILY_INVALID:
	case G_SOCKET_FAMILY_UNIX:
	default:
		break;
	}

	return NULL;
}
